-- ===========================================================================
--	HUD Launch Bar
--	Controls raising full-screen and "choosers"
-- ===========================================================================

-- ===========================================================================
--	SOME OF THIS CONTENT IS TAKEN FROM CQUI WHICH FALLS UNDER THE MIT LICENSE WHICH CAN IS CONTAINED IN THIS MOD
-- ===========================================================================

include( "GameCapabilities" );
include("InstanceManager");

local m_numTreesOpen:number = 0;
local isTechTreeOpen	:boolean = false;
local isCivicsTreeOpen	:boolean = false;
local isGreatPeopleOpen	:boolean = false;
local isGreatWorksOpen	:boolean = false;
local isReligionOpen	:boolean = false;
local isGovernmentOpen	:boolean = false;

local m_isGreatPeopleUnlocked	:boolean = false;
local m_isGreatWorksUnlocked	:boolean = false;
local m_isReligionUnlocked		:boolean = false;
local m_isGovernmentUnlocked	:boolean = false;
ExposedMembers.DoctrinesScreenStatus = false;

local isDebug			:boolean = false;			-- Set to true to force all hook buttons to show on game start
local m_PromotionListInstanceMgr:table	= InstanceManager:new( "PromotionSelectionInstance",	"PromotionSelection",	Controls.DoctrinesScrollPanel );
local m_CompletedPromotionListInstanceMgr:table	= InstanceManager:new( "CompletedPromotionSelectionInstance",	"PromotionSelection",	Controls.DoctrinesScrollPanel );	

-- ===========================================================================
--	Callbacks
-- ===========================================================================
function OnGovernmentClick()
  local ePlayer		:number = Game.GetLocalPlayer();
  if ePlayer ~= -1 then
    localPlayer = Players[ePlayer];
    if localPlayer == nil then
      return;
    end
  else
    return;
  end
  local kCulture:table	= localPlayer:GetCulture();
  if isGovernmentOpen then
    LuaEvents.LaunchBar_CloseGovernmentPanel()
  else
    CloseAllPopups();
    if (kCulture:CivicCompletedThisTurn() and kCulture:CivicUnlocksGovernment(kCulture:GetCivicCompletedThisTurn()) and not kCulture:GovernmentChangeConsidered()) then
      -- Blocking notification that NEW GOVERNMENT is available, make sure player takes a look
      LuaEvents.LaunchBar_GovernmentOpenGovernments();
    else
      -- Normal entry to my Government
      LuaEvents.LaunchBar_GovernmentOpenMyGovernment();
    end
  end
end

-- ===========================================================================
function CloseAllPopups()
  LuaEvents.LaunchBar_CloseGreatPeoplePopup();
  LuaEvents.LaunchBar_CloseGreatWorksOverview();
  LuaEvents.LaunchBar_CloseReligionPanel();
  if isGovernmentOpen then
    LuaEvents.LaunchBar_CloseGovernmentPanel();
  end
  LuaEvents.LaunchBar_CloseTechTree();
  LuaEvents.LaunchBar_CloseCivicsTree();
end

-- ===========================================================================
function OnOpenGreatPeople()
  if isGreatPeopleOpen then
    LuaEvents.LaunchBar_CloseGreatPeoplePopup();
  else
    CloseAllPopups();
    LuaEvents.LaunchBar_OpenGreatPeoplePopup();
  end
end

-- ===========================================================================
function OnOpenGreatWorks()
  if isGreatWorksOpen then
    LuaEvents.LaunchBar_CloseGreatWorksOverview();
  else
    CloseAllPopups();
    LuaEvents.LaunchBar_OpenGreatWorksOverview();
  end
end

-- ===========================================================================
function OnOpenReligion()
  if isReligionOpen then
    LuaEvents.LaunchBar_CloseReligionPanel();
  else
    CloseAllPopups();
    LuaEvents.LaunchBar_OpenReligionPanel();
  end
end

-- ===========================================================================
function OnOpenResearch()
  if isTechTreeOpen then
    LuaEvents.LaunchBar_CloseTechTree();
  else
    CloseAllPopups();
    LuaEvents.LaunchBar_RaiseTechTree();
  end
end

-- ===========================================================================
function OnOpenCulture()
  if isCivicsTreeOpen then
    LuaEvents.LaunchBar_CloseCivicsTree();
  else
    CloseAllPopups();
    LuaEvents.LaunchBar_RaiseCivicsTree();
  end
end

-- ===========================================================================
-- CQUI: Moved here from toppanel.lua since we moved the reports button here
function OnToggleReportsScreen()
  local pReportsScreen :table = ContextPtr:LookUpControl( "/InGame/ReportScreen" );
  if pReportsScreen == nil then
    UI.DataError("Unable to toggle Reports Screen.  Not found in '/InGame/ReportScreen'.");
    return;
  end
  if pReportsScreen:IsHidden() then
    LuaEvents.TopPanel_OpenReportsScreen();
  else
    LuaEvents.TopPanel_CloseReportsScreen();
  end
end

-- ===========================================================================
-- DOCTRINES
-- ===========================================================================
function OnOpenDoctrines()
	print(ExposedMembers.DoctrinesScreenStatus);
	LuaEvents.LaunchBar_CloseTechTree();
	LuaEvents.LaunchBar_CloseCivicsTree();
	LuaEvents.LaunchBar_CloseReligionPanel();
	LuaEvents.LaunchBar_CloseGreatWorksOverview();
	LuaEvents.LaunchBar_CloseGreatPeoplePopup();
	m_width, m_height	= UIManager:GetScreenSizeVal();
	if m_width ~= nil then
		if m_width <= 1400 then
			Controls.ButtonInfantryDoctrine:SetSizeX(140);
	
			Controls.ButtonCavalryDoctrine:SetSizeX(140);
			Controls.ButtonReconDoctrine:SetSizeX(140);
			Controls.ButtonRangedDoctrine:SetSizeX(140);
			Controls.ButtonNavalDoctrine:SetSizeX(140);
			Controls.ButtonAirDoctrine:SetSizeX(140);
		
			Controls.ButtonInfantryDoctrine:SetOffsetX(-290);
			Controls.ButtonCavalryDoctrine:SetOffsetX(10);
			Controls.ButtonReconDoctrine:SetOffsetX(-140);
			Controls.ButtonRangedDoctrine:SetOffsetX(160);
			Controls.ButtonNavalDoctrine:SetOffsetX(310);
			Controls.ButtonAirDoctrine:SetOffsetX(460);
		end
		if m_width <= 1200 then
			Controls.ButtonInfantryDoctrine:SetOffsetY(44);
			Controls.ButtonCavalryDoctrine:SetOffsetY(44);
			Controls.ButtonReconDoctrine:SetOffsetY(44);
			Controls.ButtonRangedDoctrine:SetOffsetY(44);
			Controls.ButtonNavalDoctrine:SetOffsetY(44);
			Controls.ButtonAirDoctrine:SetOffsetY(44);
		
			Controls.XPTitleHolder:SetOffsetY(80);
			Controls.DoctrineOverallDescription:SetHide(true);
		end
	end
	if ExposedMembers.DoctrinesScreenStatus == true then
		OnCloseDoctrines();
	else
		ExposedMembers.DoctrinesScreenStatus = true;
		UI.PlaySound("UI_Screen_Open");
		Controls.DoctrineScreen:SetHide(false);
		OnInfantryDoctrine();
	end
end

function OnCloseDoctrines()
	UI.PlaySound("UI_Screen_Close");
	Controls.DoctrineScreen:SetHide(true);
	ExposedMembers.DoctrinesScreenStatus = false;
	m_PromotionListInstanceMgr:ResetInstances();
end
function CloseOutDoctrines()
	Controls.SelectInfantryDoctrine:SetHide(true);
	Controls.SelectReconDoctrine:SetHide(true);
	Controls.SelectCavalryDoctrine:SetHide(true);
	Controls.SelectRangedDoctrine:SetHide(true);
	Controls.SelectNavalDoctrine:SetHide(true);
	Controls.SelectAirDoctrine:SetHide(true);
end
-- DOCTRINE SELECTED

function OnInfantryDoctrine()
	ExposedMembers.DoctrinesTypeStatus = 'Infantry';
	CloseOutDoctrines();
	Controls.SelectInfantryDoctrine:SetHide(false);
	DoctrinesDisplay();
end

function OnCavalryDoctrine()
	ExposedMembers.DoctrinesTypeStatus = 'Cavalry';
	CloseOutDoctrines();
	Controls.SelectCavalryDoctrine:SetHide(false);
	DoctrinesDisplay();
end

function OnReconDoctrine()
	ExposedMembers.DoctrinesTypeStatus = 'Recon';
	CloseOutDoctrines();
	Controls.SelectReconDoctrine:SetHide(false);
	DoctrinesDisplay();
end

function OnRangedDoctrine()
	ExposedMembers.DoctrinesTypeStatus = 'Ranged';
	CloseOutDoctrines();
	Controls.SelectRangedDoctrine:SetHide(false);
	DoctrinesDisplay();
end

function OnNavalDoctrine()
	ExposedMembers.DoctrinesTypeStatus = 'Naval';
	CloseOutDoctrines();
	Controls.SelectNavalDoctrine:SetHide(false);
	DoctrinesDisplay();
end

function OnAirDoctrine()
	ExposedMembers.DoctrinesTypeStatus = 'Air';
	CloseOutDoctrines();
	Controls.SelectAirDoctrine:SetHide(false);
	DoctrinesDisplay();
end
function levelsTotal()

end
function DoctrinesDisplay()
	m_PromotionListInstanceMgr:ResetInstances();
	m_CompletedPromotionListInstanceMgr:ResetInstances();
	m_uiNodes = {};
	doctrineTypeLetter = "";
	numPromotions = 0;
	playerID = Game.GetLocalPlayer();
	infantryDoctrineTotal = 0;
	cavalryDoctrineTotal = 0;
	reconDoctrineTotal = 0;
	rangedDoctrineTotal = 0;
	navalDoctrineTotal = 0;
	airDoctrineTotal = 0;
	local MyGlobalsRef : table = {};
		LuaEvents.GetMyGlobals(MyGlobalsRef);
	if ExposedMembers.doctrineLevelTableEM ~= nil then
					for i, doctrineLevelItem in ipairs(ExposedMembers.doctrineLevelTableEM) do
									rightGroup = true;
									for j, doctrineLevelItem in ipairs(doctrineLevelItem) do
											
											if j == 1 and (doctrineLevelItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											-- Looping through all ArmyUnits
											if j == 2 and rightGroup then
												infantryDoctrineTotal = infantryDoctrineTotal + doctrineLevelItem;
											end
											if j == 3 and rightGroup then
												infantryDoctrineTotal = infantryDoctrineTotal + doctrineLevelItem;
											end
											if j == 4 and rightGroup then
												infantryDoctrineTotal = infantryDoctrineTotal + doctrineLevelItem;
											end
											if j == 5 and rightGroup then
												cavalryDoctrineTotal = cavalryDoctrineTotal + doctrineLevelItem;
											end
											if j == 6 and rightGroup then
												cavalryDoctrineTotal = cavalryDoctrineTotal + doctrineLevelItem;
											end
											if j == 7 and rightGroup then
												cavalryDoctrineTotal = cavalryDoctrineTotal + doctrineLevelItem;
											end
											if j == 8 and rightGroup then
												reconDoctrineTotal = reconDoctrineTotal + doctrineLevelItem;
											end
											if j == 9 and rightGroup then
												reconDoctrineTotal = reconDoctrineTotal + doctrineLevelItem;
											end
											if j == 10 and rightGroup then
												reconDoctrineTotal = reconDoctrineTotal + doctrineLevelItem;
											end
											if j == 11 and rightGroup then
												rangedDoctrineTotal = rangedDoctrineTotal + doctrineLevelItem;
											end
											if j == 12 and rightGroup then
												rangedDoctrineTotal = rangedDoctrineTotal + doctrineLevelItem;
											end
											if j == 13 and rightGroup then
												rangedDoctrineTotal = rangedDoctrineTotal + doctrineLevelItem;
											end
											if j == 14 and rightGroup then
												navalDoctrineTotal = navalDoctrineTotal + doctrineLevelItem;
											end
											if j == 15 and rightGroup then
												navalDoctrineTotal = navalDoctrineTotal + doctrineLevelItem;
											end
											if j == 16 and rightGroup then
												navalDoctrineTotal = navalDoctrineTotal + doctrineLevelItem;
											end
											if j == 17 and rightGroup then
												airDoctrineTotal = airDoctrineTotal + doctrineLevelItem;
											end
											if j == 18 and rightGroup then
												airDoctrineTotal = airDoctrineTotal + doctrineLevelItem;
											end
											if j == 19 and rightGroup then
												airDoctrineTotal = airDoctrineTotal + doctrineLevelItem;
											end
									end								
					end
	end
	infantryDoctrineCost = (infantryDoctrineTotal + 1) * 50 + (infantryDoctrineTotal * 40);
	print("IDT: " .. infantryDoctrineTotal);
	cavalryDoctrineCost = (cavalryDoctrineTotal + 1) * 50 + (cavalryDoctrineTotal * 40);
	reconDoctrineCost = (reconDoctrineTotal + 1) * 50 + (reconDoctrineTotal * 40);
	rangedDoctrineCost = (rangedDoctrineTotal + 1) * 50 + (rangedDoctrineTotal * 40);
	navalDoctrineCost = (navalDoctrineTotal + 1) * 50 + (navalDoctrineTotal * 40);
	airDoctrineCost = (airDoctrineTotal + 1) * 50 + (airDoctrineTotal * 40);
	xpDisplay(infantryDoctrineCost, cavalryDoctrineCost, reconDoctrineCost, rangedDoctrineCost, navalDoctrineCost, airDoctrineCost);

	if ExposedMembers.DoctrinesTypeStatus == 'Infantry' then
		doctrineTypeLetter = "I";
	end
	if ExposedMembers.DoctrinesTypeStatus == 'Cavalry' then
		doctrineTypeLetter = "C";
	end
	if ExposedMembers.DoctrinesTypeStatus == 'Recon' then
		doctrineTypeLetter = "S";
	end
	if ExposedMembers.DoctrinesTypeStatus == 'Ranged' then
		doctrineTypeLetter = "R";
	end
	if ExposedMembers.DoctrinesTypeStatus == 'Naval' then
		doctrineTypeLetter = "N";
	end
	if ExposedMembers.DoctrinesTypeStatus == 'Air' then
		doctrineTypeLetter = "A";
	end
	for row in GameInfo.UnitPromotions() do
		promotionType = row.UnitPromotionType;
		validPromotion = false;
		validMilitary = false;
		validDoctrine = false;
		validInfantry = false;
		
		for i = 1, #promotionType do
			local c = promotionType:sub(i,i)
			-- 6th ICON letter should be unique
			if c == "M" and i == 12 then 
				validMilitary = true;
			end
			if c == "D" and i == 13 then 
				validDoctrine = true;
			end  
			if c == doctrineTypeLetter and i == 11 then 
				validInfantry = true;
			end
			if validMilitary and validDoctrine and validInfantry then
				validPromotion = true;
				break;
			end
		end
		if validPromotion == true then
			doctrineTotal = 0;
			m_uiNodes[promotionType] = row;
			print("#Doctrines = " .. promotionType);
			local promotionInstance;
			hasPromotion = false;
			--if hasPromotion then
			--	promotionInstance = m_CompletedPromotionListInstanceMgr:GetInstance();
			--else
			playerID = Game.GetLocalPlayer();
			for i, promotionItem in ipairs(ExposedMembers.promotionTableEM) do
									rightGroup = true;
									print('#Doctrines Looping through Table i');
									for j, promotionItem in ipairs(promotionItem) do
											print('#Doctrines Looping through Table j');
											if j == 1 and (promotionItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
												print('#Doctrines wrong group');
											end
											-- Looping through all ArmyUnits
											if j == 2 and rightGroup and promotionItem == row.UnitPromotionType then
												hasPromotion = true;
												print('#Doctrines has promotion true');
												break;									
											end
									end								
			end
			if hasPromotion then
				promotionInstance = m_CompletedPromotionListInstanceMgr:GetInstance();
			else
				promotionInstance = m_PromotionListInstanceMgr:GetInstance();
			end -- 2* 400-
			if m_width ~= nil then
				if m_width <= 1200 then
					promotionInstance.PromotionSelection:SetOffsetVal(((row.Level - 1) * 352) + 5, (((row.Column - 1) * 200 + 130) + ((row.Level - 1) * (400 - (100 * (row.Level - 2))))));
				elseif m_width >= 1200 and m_width <= 1400 then
					promotionInstance.PromotionSelection:SetOffsetVal(((row.Level - 1) * 352) + 250, (((row.Column - 1) * 200 + 130) + ((row.Level - 1) * (400 - (100 * (row.Level - 2))))));
				else
					promotionInstance.PromotionSelection:SetOffsetVal((row.Level * 352), (((row.Column - 1) * 200 + 130) + ((row.Level - 1) * (400 - (100 * (row.Level - 2))))));
				end
			end
			if m_width == nil then
				promotionInstance.PromotionSelection:SetOffsetVal((row.Level * 352), (((row.Column - 1) * 200 + 130) + ((row.Level - 1) * (400 - (100 * (row.Level - 2))))));
			end
			promotionInstance.PromotionName:SetText(Locale.ToUpper(row.Name));
			promotionInstance.PromotionDescription:SetText(Locale.Lookup(row.Description));
			if row.Level == 1 then
				numPromotions = numPromotions + 1;
			end
			Controls.Background:SetSizeY(numPromotions * 200 + 110);
			--Controls.ArtParchmentRippleBottom:SetOffsetY((numPromotions * 200 + 110) * -1);
			--Controls.ArtParchmentDecoBottom:SetOffsetY((numPromotions * 200 + 110 + 50) * -1);
			--local iconName = "ICON_" .. promotionDefinition.UnitPromotionType;
			--local textureOffsetX, textureOffsetY, textureSheet = IconManager:FindIconAtlas(iconName,32);
			--if (textureOffsetX ~= nil) then
			--	promotionInstance.PromotionIcon:SetTexture( textureOffsetX, textureOffsetY, textureSheet );
			--end
		
			-- Check if available in saved Table
			-- table organized : [playerid, #lvl1 Infantry, #lvl2 Infantry, #lvl3 Infantry etc...] this table needs to save (save in string?)
			-- second table : [playerid, each promotion type...] also needs to save in string?
			if ExposedMembers.doctrineLevelTableEM ~= nil then
					for i, doctrineLevelItem in ipairs(ExposedMembers.doctrineLevelTableEM) do
									rightGroup = true;
									for j, doctrineLevelItem in ipairs(doctrineLevelItem) do
										
											if j == 1 and (doctrineLevelItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											-- Looping through all ArmyUnits
											if j == 2 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Infantry' and row.Level == 1 then
												infantryLvl1Total = doctrineLevelItem;
												if infantryDoctrineCost <= ExposedMembers.infantryXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
													-- save location
													
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
													
												end
												if infantryDoctrineCost <= ExposedMembers.infantryXPTotal then
													Controls.InfantryDoctrineAvailable:SetHide(false);
												else
													Controls.InfantryDoctrineAvailable:SetHide(true);
												end
											end
											if j == 2 and rightGroup then
												if infantryDoctrineCost <= ExposedMembers.infantryXPTotal then
													Controls.InfantryDoctrineAvailable:SetHide(false);
													
												else
													Controls.InfantryDoctrineAvailable:SetHide(true);
												end
												if airDoctrineCost <= ExposedMembers.airXPTotal then
													Controls.AirDoctrineAvailable:SetHide(false);
												else
													Controls.AirDoctrineAvailable:SetHide(true);
												end
												if cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal then
													Controls.CavalryDoctrineAvailable:SetHide(false);
												else
													Controls.CavalryDoctrineAvailable:SetHide(true);
												end
												if reconDoctrineCost <= ExposedMembers.reconXPTotal then
													Controls.ReconDoctrineAvailable:SetHide(false);
												else
													Controls.ReconDoctrineAvailable:SetHide(true);
												end
												if rangedDoctrineCost <= ExposedMembers.rangedXPTotal then
													Controls.RangedDoctrineAvailable:SetHide(false);
												else
													Controls.RangedDoctrineAvailable:SetHide(true);
												end
												if navalDoctrineCost <= ExposedMembers.navalXPTotal then
													Controls.NavalDoctrineAvailable:SetHide(false);
												else
													Controls.NavalDoctrineAvailable:SetHide(true);
												end
												if infantryDoctrineCost <= ExposedMembers.infantryXPTotal or navalDoctrineCost <= ExposedMembers.navalXPTotal or airDoctrineCost <= ExposedMembers.airXPTotal or reconDoctrineCost <= ExposedMembers.reconXPTotal or cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal or rangedDoctrineCost <= ExposedMembers.rangedXPTotal then
													Controls.DoctrinesAvailableIndicator:SetHide(false);
													Controls.DoctrinesAvailableIndicator:SetToolTipString("New Doctrine Available!");
												else
													Controls.DoctrinesAvailableIndicator:SetHide(true);
													Controls.DoctrinesAvailableIndicator:SetToolTipString();
												end
											end
											if j == 3 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Infantry' and row.Level == 2 then
												infantryLvl2Total = doctrineLevelItem;
												if  infantryLvl1Total >= 2 * (infantryLvl2Total + 1) and infantryDoctrineCost <= ExposedMembers.infantryXPTotal and hasPromotion == false  then
													
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 4 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Infantry' and row.Level == 3 then
												infantryLvl3Total = doctrineLevelItem;
												if infantryLvl2Total >= 2 * (infantryLvl3Total + 1) and infantryDoctrineCost <= ExposedMembers.infantryXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 5 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Cavalry' and row.Level == 1  then
												cavalryLvl1Total = doctrineLevelItem;
												if cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
												
											end
											if j == 6 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Cavalry' and row.Level == 2 then
												cavalryLvl2Total = doctrineLevelItem;
												if  cavalryLvl1Total >= 2 * (cavalryLvl2Total + 1) and hasPromotion == false and cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 7 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Cavalry' and row.Level == 3 then
												cavalryLvl3Total = doctrineLevelItem;
												if cavalryLvl2Total >= 2 * (cavalryLvl3Total + 1) and hasPromotion == false and cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 8 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Recon' and row.Level == 1 then
												reconLvl1Total = doctrineLevelItem;
												if reconDoctrineCost <= ExposedMembers.reconXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
												
											end
											if j == 9 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Recon' and row.Level == 2 then
												reconLvl2Total = doctrineLevelItem;
												if  reconLvl1Total >= 2 * (reconLvl2Total + 1) and hasPromotion == false and reconDoctrineCost <= ExposedMembers.reconXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 10 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Recon' and row.Level == 3 then
												reconLvl3Total = doctrineLevelItem;
												if reconLvl2Total >= 2 * (reconLvl3Total + 1) and hasPromotion == false and reconDoctrineCost <= ExposedMembers.reconXPTotal then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 11 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Ranged' and row.Level == 1 then
												rangedLvl1Total = doctrineLevelItem;
												if rangedDoctrineCost <= ExposedMembers.rangedXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
												
											end
											if j == 12 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Ranged' and row.Level == 2 then
												rangedLvl2Total = doctrineLevelItem;
												if  rangedLvl1Total >= 2 * (rangedLvl2Total + 1) and hasPromotion == false and rangedDoctrineCost <= ExposedMembers.rangedXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 13 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Ranged' and row.Level == 3 then
												rangedLvl3Total = doctrineLevelItem;
												if rangedLvl2Total >= 2 * (rangedLvl3Total + 1) and hasPromotion == false and rangedDoctrineCost <= ExposedMembers.rangedXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 14 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Naval' and row.Level == 1 then
												navalLvl1Total = doctrineLevelItem;
												if navalDoctrineCost <= ExposedMembers.navalXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
												
											end
											if j == 15 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Naval' and row.Level == 2 then
												navalLvl2Total = doctrineLevelItem;
												print("Naval 1 Total: " .. navalLvl1Total);
												print("Naval 2 Total: " .. navalLvl1Total);
												if  navalLvl1Total >= 2 * (navalLvl2Total + 1) and hasPromotion == false and navalDoctrineCost <= ExposedMembers.navalXPTotal then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 16 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Naval' and row.Level == 3 then
												navalLvl3Total = doctrineLevelItem;
												if navalLvl2Total >= 2 * (navalLvl3Total + 1) and hasPromotion == false and navalDoctrineCost <= ExposedMembers.navalXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 17 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Air' and row.Level == 1 then
												airLvl1Total = doctrineLevelItem;
												if airDoctrineCost <= ExposedMembers.airXPTotal and hasPromotion == false then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
												
											end
											
											if j == 18 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Air' and row.Level == 2 then
												airLvl2Total = doctrineLevelItem;
												if  airLvl1Total >= 2 * (airLvl2Total + 1) and hasPromotion == false and airDoctrineCost <= ExposedMembers.airXPTotal  then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
											if j == 19 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Air' and row.Level == 3 then
												airLvl3Total = doctrineLevelItem;
												if airLvl2Total >= 2 * (airLvl3Total + 1) and hasPromotion == false and airDoctrineCost <= ExposedMembers.airXPTotal then
													promotionInstance.PromotionSlot:SetDisabled(false);
													promotionInstance.PromotionSlot:RegisterCallback( Mouse.eLClick, OnDoctrineClicked );
													promotionInstance.PromotionSlot:SetVoid1( row.Index );
												else
													promotionInstance.PromotionSlot:SetDisabled(true);
												end
											end
									end								
						end
			end
		end
		Controls.DoctrinesScrollPanel:CalculateInternalSize();
	end
end

function xpDisplay(infantryDoctrineCost, cavalryDoctrineCost, reconDoctrineCost, rangedDoctrineCost, navalDoctrineCost, airDoctrineCost)
	playerID = Game.GetLocalPlayer();
	for i, xpItem in ipairs(ExposedMembers.doctrineXPTableEM) do
									rightGroup = true;
									for j, xpItem in ipairs(xpItem) do
											
											if j == 1 and (xpItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											if j == 2 and rightGroup then
												ExposedMembers.infantryXPTotal = xpItem;								
											end
											if j == 3 and rightGroup then
												ExposedMembers.cavalryXPTotal = xpItem;								
											end
											if j == 4 and rightGroup then
												ExposedMembers.reconXPTotal = xpItem;								
											end
											if j == 5 and rightGroup then
												ExposedMembers.rangedXPTotal = xpItem;								
											end
											if j == 6 and rightGroup then
												ExposedMembers.navalXPTotal = xpItem;								
											end
											if j == 7 and rightGroup then
												ExposedMembers.airXPTotal = xpItem;								
											end
											if j == 2 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Infantry' then
												Controls.xpLabel:SetText(xpItem .. " / " .. infantryDoctrineCost .. " xp");	
												ExposedMembers.infantryXPTotal = xpItem;								
											end
											
											if j == 3 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Cavalry' then
												Controls.xpLabel:SetText(xpItem .. " / " .. cavalryDoctrineCost .. " xp");
												ExposedMembers.cavalryXPTotal = xpItem;										
											end
											if j == 4 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Recon' then
												
												Controls.xpLabel:SetText(xpItem .. " / " .. reconDoctrineCost .. " xp");
												ExposedMembers.reconXPTotal = xpItem;										
											end
											if j == 5 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Ranged' then
												Controls.xpLabel:SetText(xpItem .. " / " .. rangedDoctrineCost .. " xp");
												ExposedMembers.rangedXPTotal = xpItem;										
											end
											if j == 6 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Naval' then
												
												Controls.xpLabel:SetText(xpItem .. " / " .. navalDoctrineCost .. " xp");
												ExposedMembers.navalXPTotal = xpItem;										
											end
											if j == 7 and rightGroup and ExposedMembers.DoctrinesTypeStatus == 'Air' then
												Controls.xpLabel:SetText(xpItem .. " / " .. airDoctrineCost .. " xp");
												ExposedMembers.airXPTotal = xpItem;										
											end
											
									end								
					end
end

function OnDoctrineClicked(index)
	for row in GameInfo.UnitPromotions() do
		if index == row.Index then
			promotionType = row.UnitPromotionType;
			classType = row.PromotionClass;
			level = row.Level;
		end
	end
		playerID = Game.GetLocalPlayer();
		print(promotionType);
		print(classType);
		storePromotion(playerID, promotionType, classType);
		addLevelToPlayer(playerID, classType, level);
		DoctrinesDisplay();
end



function storePromotion(playerID, doctrineType, doctrineClass)
		local MyGlobalsRef : table = {};
		LuaEvents.GetMyGlobals(MyGlobalsRef);
		newPromotionTable = {};
		print("#Doctrines Storing Promotion");
		table.insert(newPromotionTable, 1, playerID);
		table.insert(newPromotionTable, 2, doctrineType);
		print(doctrineType);
		table.insert(newPromotionTable, 3, doctrineClass);
		table.insert(ExposedMembers.promotionTableEM, 1, newPromotionTable);

		MyGlobalsRef.MyGlobals.globalPromotionTable = ExposedMembers.promotionTableEM;
		savePromotion();

end

function addLevelToPlayer(playerID, doctrineType, level)
	if ExposedMembers.doctrineXPTableEM ~= nil then
					for i, xpItem in ipairs(ExposedMembers.doctrineLevelTableEM) do
									rightGroup = true;
									for j, xpItem in ipairs(xpItem) do
											if j == 1 and (xpItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											-- Looping through all ArmyUnits
											if j == 2 and rightGroup and doctrineType == 'PROMOTION_CLASS_MELEE' and level == 1 then
												newXP = 0;
												newXP = xpItem + 1;
												print(newXP);
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;
												print("store " .. ExposedMembers.doctrineLevelTableEM[i][j]);										
											end
											if j == 3 and rightGroup and doctrineType == 'PROMOTION_CLASS_MELEE' and level == 2 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;
												print("store " .. ExposedMembers.doctrineLevelTableEM[i][j]);												
											end
											if j == 4 and rightGroup and doctrineType == 'PROMOTION_CLASS_MELEE' and level == 3 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 5 and rightGroup and doctrineType == 'PROMOTION_CLASS_LIGHT_CAVALRY' and level == 1 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;									
											end
											if j == 6 and rightGroup and doctrineType == 'PROMOTION_CLASS_LIGHT_CAVALRY' and level == 2 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;									
											end
											if j == 7 and rightGroup and doctrineType == 'PROMOTION_CLASS_LIGHT_CAVALRY' and level == 3 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;									
											end
											if j == 8 and rightGroup and doctrineType == 'PROMOTION_CLASS_RECON' and level == 1 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 9 and rightGroup and doctrineType == 'PROMOTION_CLASS_RECON' and level == 2 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 10 and rightGroup and doctrineType == 'PROMOTION_CLASS_RECON' and level == 3 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 11 and rightGroup and doctrineType == 'PROMOTION_CLASS_RANGED' and level == 1 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;									
											end
											if j == 12 and rightGroup and doctrineType == 'PROMOTION_CLASS_RANGED' and level == 2 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;									
											end 
											if j == 13 and rightGroup and doctrineType == 'PROMOTION_CLASS_RANGED' and level == 3 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;									
											end
											if j == 14 and rightGroup and doctrineType == 'PROMOTION_CLASS_NAVAL_MELEE' and level == 1 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 15 and rightGroup and doctrineType == 'PROMOTION_CLASS_NAVAL_MELEE' and level == 2 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 16 and rightGroup and doctrineType == 'PROMOTION_CLASS_NAVAL_MELEE' and level == 3 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 17 and rightGroup and doctrineType == 'PROMOTION_CLASS_AIR_FIGHTER' and level == 1 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 18 and rightGroup and doctrineType == 'PROMOTION_CLASS_AIR_FIGHTER' and level == 2 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
											if j == 19 and rightGroup and doctrineType == 'PROMOTION_CLASS_AIR_FIGHTER' and level == 3 then
												newXP = 0;
												newXP = xpItem + 1;
												ExposedMembers.doctrineLevelTableEM[i][j]=newXP;										
											end
									end								
					end
	end
	saveLevelTable();
end

function saveLevelTable()
		savedString = "";
		if ExposedMembers.doctrineLevelTableEM ~= nil then
				for i, xpGroupItem in ipairs(ExposedMembers.doctrineLevelTableEM) do
								-- Looping through All Army Groups
								rightGroup = true;
								savedString = savedString .. "!";
								for j, xpItem in ipairs(xpGroupItem) do
										
										if j == 1 then
											savedString = savedString .. "@" .. xpItem .. "@";
										end
										-- Looping through all ArmyUnits
										if j >= 2 and rightGroup then
											savedString = savedString .. "#" .. xpItem .. "#";
										end
								end
								savedString = savedString .. "!";
					end
			print(savedString);
			GameConfiguration.SetValue ("doctrineLevelTable", savedString);
			doctrineLevelTable = GameConfiguration.GetValue ("doctrineLevelTable");
			print("Level Table: " .. doctrineLevelTable);
		end
end

function savePromotion()
		savedString = "";
		if ExposedMembers.promotionTableEM ~= nil then
				for i, xpGroupItem in ipairs(ExposedMembers.promotionTableEM) do
								-- Looping through All Army Groups
								rightGroup = true;
								savedString = savedString .. "!";
								for j, xpItem in ipairs(xpGroupItem) do
										
										if j == 1 then
											savedString = savedString .. "@" .. xpItem .. "@";
										end
										-- Looping through all ArmyUnits
										if j == 2 and rightGroup then
											savedString = savedString .. "#" .. xpItem .. "#";
										end
										if j == 3 and rightGroup then
											savedString = savedString .. "$" .. xpItem .. "$";
										end
								end
								savedString = savedString .. "!";
					end
			print(savedString);
			GameConfiguration.SetValue ("doctrinePromotionTable", savedString);
			doctrinePromotionTable = GameConfiguration.GetValue ("doctrinePromotionTable");
			print("Level Table: " .. doctrinePromotionTable);
		end
end
-- ===========================================================================
function OnOpenOldCityStates()
  LuaEvents.TopPanel_OpenOldCityStatesPopup();
end

function SetCivicsTreeOpen()
  isCivicsTreeOpen = true;
  OpenTree();
end

function SetTechTreeOpen()
  isTechTreeOpen = true;
  OpenTree();
end

function SetGreatPeopleOpen()
  isGreatPeopleOpen = true;
  OpenTree();
end

function SetGreatWorksOpen()
  isGreatWorksOpen = true;
  OpenTree();
end

function SetReligionOpen()
  isReligionOpen = true;
  OpenTree();
end

function SetGovernmentOpen()
  isGovernmentOpen = true;
  OpenTree();
end

function SetCivicsTreeClosed()
  isCivicsTreeOpen = false;
  CloseTree();
end

function SetTechTreeClosed()
  isTechTreeOpen = false;
  CloseTree();
end

function SetGreatPeopleClosed()
  isGreatPeopleOpen = false;
  CloseTree();
end

function SetGreatWorksClosed()
  isGreatWorksOpen = false;
  CloseTree();
end

function SetReligionClosed()
  isReligionOpen = false;
  CloseTree();
end

function SetGovernmentClosed()
  isGovernmentOpen = false;
  CloseTree();
end

-- ===========================================================================
--	Lua Event
--	Tutorial system is requesting any screen openned, to be closed.
-- ===========================================================================
function OnTutorialCloseAll()
  CloseAllPopups();
end

-- ===========================================================================
--	Game Engine Event
-- ===========================================================================
function OnInterfaceModeChanged(eOldMode:number, eNewMode:number)
  if eNewMode == InterfaceModeTypes.VIEW_MODAL_LENS then
    ContextPtr:SetHide(true);
  end
  if eOldMode == InterfaceModeTypes.VIEW_MODAL_LENS then
    ContextPtr:SetHide(false);
  end
end


-- ===========================================================================
--	Refresh Data and View
-- ===========================================================================
function RealizeHookVisibility()

  if (HasCapability("CAPABILITY_TECH_TREE")) then
    Controls.ScienceButton:SetHide(false);
    Controls.ScienceBolt:SetHide(false);
  else
    if (not isDebug) then
      Controls.ScienceButton:SetHide(true);
      Controls.ScienceBolt:SetHide(true);
    end
  end

  if (HasCapability("CAPABILITY_CIVICS_TREE")) then
    Controls.CultureButton:SetHide(false);
    Controls.CultureBolt:SetHide(false);
  else
    if (not isDebug) then
      Controls.CultureButton:SetHide(true);
      Controls.CultureBolt:SetHide(true);
    end
  end

  if (m_isGreatPeopleUnlocked and HasCapability("CAPABILITY_GREAT_PEOPLE_VIEW")) then
    Controls.GreatPeopleButton:SetHide(false);
    Controls.GreatPeopleBolt:SetHide(false);
  else
    if (not isDebug) then
      Controls.GreatPeopleButton:SetHide(true);
      Controls.GreatPeopleBolt:SetHide(true);
    end
  end

  if (m_isReligionUnlocked and HasCapability("CAPABILITY_RELIGION_VIEW")) then
    Controls.ReligionButton:SetHide(false);
    Controls.ReligionBolt:SetHide(false);
  else
    if (not isDebug) then
      Controls.ReligionButton:SetHide(true);
      Controls.ReligionBolt:SetHide(true);
    end
  end

  if (m_isGreatWorksUnlocked and HasCapability("CAPABILITY_GREAT_WORKS_VIEW")) then
    Controls.GreatWorksButton:SetHide(false);
    Controls.GreatWorksBolt:SetHide(false);
  else
    if (not isDebug) then
      Controls.GreatWorksButton:SetHide(true);
      Controls.GreatWorksBolt:SetHide(true);
    end
  end

  if (m_isGovernmentUnlocked and HasCapability("CAPABILITY_GOVERNMENTS_VIEW")) then
    Controls.GovernmentButton:SetHide(false);
    Controls.GovernmentBolt:SetHide(true);
  else
    if (not isDebug) then
      Controls.GovernmentButton:SetHide(true);
      Controls.GovernmentBolt:SetHide(true);
    end
  end

  RefreshView();
end

--	Note on hook show/hide functionality:
--	We do not serialize any of this data, but instead we will check gamestate OnTurnBegin to determine which hooks should be shown.
--	Once the show/hide flags have been set, we return from the function before performing the checks again.
--	For all of the hooks that start in a hidden state, there are two functions needed to correctly capture the event to show/hide the hook:
--	1/2) A function for capturing the event as it happens during a turn of gameplay
--	2/2) A function to check gamestate OnTurnBegin

-- *****************************************************************************
--	Religion Hook
--	1/2) OnFaithChanged - triggered off of the FaithChanged game event
function OnFaithChanged()
  if (m_isReligionUnlocked) then
    return;
  end
  m_isReligionUnlocked = true;
  RealizeHookVisibility();
end

--	2/2) RefreshReligion - this function checks to see if any faith has been earned
function RefreshReligion()
  local ePlayer:number = Game.GetLocalPlayer();
  if ePlayer == -1 then
    -- Likely auto playing.
    return;
  end
  if m_isReligionUnlocked then
    return;
  end
  local localPlayer = Players[ePlayer];
  local playerReligion		:table	= localPlayer:GetReligion();
  local faithYield			:number = playerReligion:GetFaithYield();
  if (faithYield > 0) then
    m_isReligionUnlocked = true;
  end
  RealizeHookVisibility();
end

-- *****************************************************************************
--	Great Works Hook
--	1/2) OnGreatWorkCreated - triggered off of the GreatWorkCreated game event
--	*Note - a great work can be added and then traded away/ moved.  I think we should still allow the hook to stay
--	open in this case.  I think it would be strange behavior to have the hook be made available and then removed.
function OnGreatWorkCreated()
  if (m_isGreatWorksUnlocked) then
    return;
  end
  m_isGreatWorksUnlocked = true;
  RealizeHookVisibility();
end

-- also need to capture when a deal has left us with a great work
function OnDiplomacyDealEnacted()
  if (not m_isGreatWorksUnlocked) then
    RefreshGreatWorks();
  end
end

--	2/2) RefreshGreatWorks - go through each building checking for GW slots, then query that slot for a slotted great work
function RefreshGreatWorks()
  local ePlayer:number = Game.GetLocalPlayer();
  if ePlayer == -1 then
    -- Likely auto playing.
    return;
  end
  if m_isGreatWorksUnlocked then
    return;
  end

  localPlayer = Players[ePlayer];
  local pCities:table = localPlayer:GetCities();
  for i, pCity in pCities:Members() do
    if pCity ~= nil and pCity:GetOwner() == ePlayer then
      local pCityBldgs:table = pCity:GetBuildings();
      for buildingInfo in GameInfo.Buildings() do
        local buildingIndex:number = buildingInfo.Index;
        if(pCityBldgs:HasBuilding(buildingIndex)) then
          local numSlots:number = pCityBldgs:GetNumGreatWorkSlots(buildingIndex);
          if (numSlots ~= nil and numSlots > 0) then
            for slotIndex=0, numSlots - 1 do
              local greatWorkIndex:number = pCityBldgs:GetGreatWorkInSlot(buildingIndex, slotIndex);
              if (greatWorkIndex ~= -1) then
                m_isGreatWorksUnlocked = true;
                break;
              end
            end
          end
        end
      end
    end
  end
  RealizeHookVisibility();
end

function RefreshGreatPeople()
  local ePlayer:number = Game.GetLocalPlayer();
  if ePlayer == -1 then
    -- Likely auto playing.
    return;
  end
  if m_isGreatPeopleUnlocked then
    return;
  end

  -- Show button if we have any great people in the game
  for greatPerson in GameInfo.GreatPersonIndividuals() do
    m_isGreatPeopleUnlocked = true;
    break;
  end
  RealizeHookVisibility();
end

-- *****************************************************************************
--	Government Hook
--	1/2) OnCivicCompleted - triggered off of the CivicCompleted event - check to see if the unlocked civic unlocked our first policy
function OnCivicCompleted(player:number, civic:number, isCanceled:boolean)
  local ePlayer:number = Game.GetLocalPlayer();
  if ePlayer == -1 then
    return;
  end
  if(not m_isGovernmentUnlocked) then
    local playerCulture:table = Players[ePlayer]:GetCulture();
    if (playerCulture:GetNumPoliciesUnlocked() > 0) then
      m_isGovernmentUnlocked = true;
      RealizeHookVisibility();
    end
  end
end

--	2/2) RefreshGovernment - Check against the number of policies unlocked
function RefreshGovernment()
  local ePlayer:number = Game.GetLocalPlayer();
  if ePlayer == -1 then
    -- Likely auto playing.
    return;
  end

  -- GOVERNMENT BUTTON
  local playerCulture:table = Players[ePlayer]:GetCulture();
  if (playerCulture:GetNumPoliciesUnlocked() <= 0) then
    Controls.GovernmentButton:SetToolTipString(Locale.Lookup("LOC_GOVERNMENT_DOESNT_UNLOCK"));
    Controls.GovernmentButton:GetTextControl():SetColor(0xFF666666);
  else
    m_isGovernmentUnlocked = true;
    Controls.GovernmentButton:SetHide(false);
    Controls.GovernmentBolt:SetHide(false);
    Controls.GovernmentButton:SetToolTipString(Locale.Lookup("LOC_GOVERNMENT_MANAGE_GOVERNMENT_AND_POLICIES"));
    local bPoliciesChanged = playerCulture:PolicyChangeMade();
    local bInAnarchy = playerCulture:IsInAnarchy();
    if (bInAnarchy == true) then
      Controls.GovernmentButton:SetToolTipString("[COLOR_RED]".. Locale.Lookup("LOC_GOVERNMENT_ANARCHY_NAME") .. "[ENDCOLOR]");
      Controls.GovernmentButton:SetDisabled(true);
      Controls.GovernmentIcon:SetColorByName("Civ6Red");
    elseif (bPoliciesChanged == true) then
      Controls.GovernmentButton:SetToolTipString(Locale.Lookup("LOC_GOVERNMENT_MANAGE_GOVERNMENT_AND_POLICIES"));
      Controls.GovernmentButton:SetDisabled(false);
      Controls.GovernmentIcon:SetColorByName("White");
    else
      Controls.GovernmentIcon:SetColorByName("White");
      local iPolicyUnlockCost = playerCulture:GetCostToUnlockPolicies();
      if (iPolicyUnlockCost == 0) then
        Controls.PoliciesAvailableIndicator:SetHide(false);
        Controls.PoliciesAvailableIndicator:SetToolTipString(Locale.Lookup("LOC_HUD_GOVT_FREE_CHANGES"));
      else
        Controls.PoliciesAvailableIndicator:SetHide(true);
        Controls.PoliciesAvailableIndicator:SetToolTipString();
      end
      Controls.GovernmentButton:SetDisabled(false);
    end
  end
  RealizeHookVisibility();
end

-- ===========================================================================
function RefreshView()
  -- The Launch Bar width should accomodate how many hooks are currently in the stack.
  Controls.ButtonStack:CalculateSize();
  Controls.ButtonStack:ReprocessAnchoring();
  Controls.LaunchBacking:SetSizeX(Controls.ButtonStack:GetSizeX()+116);
  Controls.LaunchBackingTile:SetSizeX(Controls.ButtonStack:GetSizeX()-20);
  Controls.LaunchBarDropShadow:SetSizeX(Controls.ButtonStack:GetSizeX());
  -- When we change size of the LaunchBar, we send this LuaEvent to the Diplomacy Ribbon, so that it can change scroll width to accommodate it
  LuaEvents.LaunchBar_Resize(Controls.ButtonStack:GetSizeX());
end

-- ===========================================================================
function OnTurnBegin()
  local localPlayer				= Players[Game.GetLocalPlayer()];
  if (localPlayer == nil) then
    return;
  end
  local playerTechs				= localPlayer:GetTechs();
  local currentTechID		:number = playerTechs:GetResearchingTech();

  if(currentTechID >= 0) then
    local progress			:number = playerTechs:GetResearchProgress(currentTechID);
    local cost				:number	= playerTechs:GetResearchCost(currentTechID);

    Controls.ScienceMeter:SetPercent(progress/cost);
    Controls.ScienceTurnsLeft:SetText(playerTechs:GetTurnsLeft());
  else
    Controls.ScienceMeter:SetPercent(0);
    Controls.ScienceTurnsLeft:SetText("");
  end

  local techInfo:table = GameInfo.Technologies[currentTechID];
  if (techInfo ~= nil) then
    local textureString = "ICON_" .. techInfo.TechnologyType;
    local textureOffsetX, textureOffsetY, textureSheet = IconManager:FindIconAtlas(textureString,38);
    if textureSheet ~= nil then
      Controls.ResearchIcon:SetTexture(textureOffsetX, textureOffsetY, textureSheet);
    end
  end

  local playerCivics				= localPlayer:GetCulture();
  local currentCivicID    :number = playerCivics:GetProgressingCivic();

  if(currentCivicID >= 0) then
    local civicProgress			:number = playerCivics:GetCulturalProgress(currentCivicID);
    local civicCost				:number	= playerCivics:GetCultureCost(currentCivicID);

    Controls.CultureMeter:SetPercent(civicProgress/civicCost);
    Controls.CultureTurnsLeft:SetText(playerCivics:GetTurnsLeft());
  else
    Controls.CultureMeter:SetPercent(0);
    Controls.CultureTurnsLeft:SetText("");
  end

  local CivicInfo:table = GameInfo.Civics[currentCivicID];
  if (CivicInfo ~= nil) then
    local civictextureString = "ICON_" .. CivicInfo.CivicType;
    local civictextureOffsetX, civictextureOffsetY, civictextureSheet = IconManager:FindIconAtlas(civictextureString,38);
    if civictextureSheet ~= nil then
      Controls.CultureIcon:SetTexture(civictextureOffsetX, civictextureOffsetY, civictextureSheet);
    end
  end

  RefreshGovernment();
  RefreshGreatWorks();
  RefreshGreatPeople();
  RefreshReligion();
  RefreshView();
end

function OpenTree()
  m_numTreesOpen = m_numTreesOpen+1;
  local screenX, screenY:number = UIManager:GetScreenSizeVal();
  if screenY <= 850 then
    Controls.LaunchContainer:SetOffsetY(-35);
    Controls.ScienceHookWithMeter:SetOffsetY(-5);
    Controls.CultureHookWithMeter:SetOffsetY(-5);
  end
end

function CloseTree()
  m_numTreesOpen = m_numTreesOpen-1;
  if(m_numTreesOpen < 0 )then
    m_numTreesOpen = 0;
  end
  if m_numTreesOpen == 0 then
    Controls.LaunchContainer:SetOffsetY(-5);
    Controls.ScienceHookWithMeter:SetOffsetY(25);
    Controls.CultureHookWithMeter:SetOffsetY(25);
  end
end

-- ===========================================================================
function OnToggleResearchPanel(hideResearch)
  Controls.ScienceHookWithMeter:SetHide(not hideResearch);
end

function OnToggleCivicPanel(hideResearch)
  Controls.CultureHookWithMeter:SetHide(not hideResearch);
end

-- Reset the hooks when the player changes for hotseat.
function OnLocalPlayerChanged()
  m_isGreatPeopleUnlocked	= false;
  m_isGreatWorksUnlocked	= false;
  m_isReligionUnlocked	= false;
  m_isGovernmentUnlocked	= false;
  RefreshGovernment();
  RefreshGreatPeople();
  RefreshGreatWorks();
  RefreshReligion();
end

-- ===========================================================================
--	Input
--	UI Event Handler
-- ===========================================================================
function KeyUpHandler( key:number )
	if ExposedMembers.DoctrinesScreenStatus == true then
		if key == Keys.VK_ESCAPE then
			OnCloseDoctrines();
			ExposedMembers.DoctrinesScreenStatus = false;
			return true;
		end
	end
	return false;
end
function OnInputHandler( pInputStruct:table )
	local uiMsg = pInputStruct:GetMessageType();
	if uiMsg == KeyEvents.KeyUp then return KeyUpHandler( pInputStruct:GetKey() ); end	
	return false;
end

-- ===========================================================================
function OnCombat(combatMembers)
		savedString = "";
		if ExposedMembers.doctrineXPTableEM ~= nil then
				for i, xpGroupItem in ipairs(ExposedMembers.doctrineXPTableEM) do
								
								rightGroup = true;
								savedString = savedString .. "!";
								for j, xpItem in ipairs(xpGroupItem) do
										
										if j == 1 then
											savedString = savedString .. "@" .. xpItem .. "@";
										end
										
										if j >= 2 then
											savedString = savedString .. "#" .. xpItem .. "#";
										end
								end
								savedString = savedString .. "!";
					end
			print(savedString);
			GameConfiguration.SetValue ("doctrineXPTable", savedString);
			doctrineXPTable = GameConfiguration.GetValue ("doctrineXPTable");
			print("XP Table: " .. doctrineXPTable);
		end
		DoctrinesDisplay();

end
Events.Combat.Add( OnCombat );

function selectRandomDoctrine(playerID, doctrineTypeLetter, level)
	print("Selecting Random Promotion");
	
	local promotionChosen = false;
	local random_num7 = math.random( 1, 8 );
	local random_num4 = math.random( 1, 4 );
	local random_num2 = math.random( 1, 2 );
	for row in GameInfo.UnitPromotions() do
		local hasPromotion = false;
		for i, promotionItem in ipairs(ExposedMembers.promotionTableEM) do
									rightGroup = true;
									for j, promotionItem in ipairs(promotionItem) do
											if j == 1 and (promotionItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											-- Looping through all ArmyUnits
											if j == 2 and rightGroup and promotionItem == row.UnitPromotionType then
												hasPromotion = true;
												break;									
											end
									end								
		end
		promotionType = row.UnitPromotionType;
		validPromotion = false;
		validMilitary = false;
		validDoctrine = false;
		validInfantry = false;
		
		for i = 1, #promotionType do
			local c = promotionType:sub(i,i)
			-- 6th ICON letter should be unique
			if c == "M" and i == 12 then 
				validMilitary = true;
			end
			if c == "D" and i == 13 then 
				validDoctrine = true;
			end  
			if c == doctrineTypeLetter and i == 11 then 
				validInfantry = true;
			end
			if validMilitary and validDoctrine and validInfantry then
				validPromotion = true;
				break;
			end
		end
		if validPromotion == true then
			print("Valid Promotion");
			print(hasPromotion);
			print(promotionChosen);
			print(level);
			print(row.Level);
			print(row.Column);
			print(random_num7);
			print(random_num2);
			if hasPromotion and (row.Column == random_num4 or row.Column == random_num2 or row.Column == random_num7) then
				print("Already has Promotion: " .. row.UnitPromotionType);
				random_num7 = math.random( 1, 8 );
				random_num4 = math.random( 1, 4 );
				random_num2 = math.random( 1, 2 );
			end
			if row.Level == level and hasPromotion == false and promotionChosen == false then
				print("Matching Level");
				if level == 2 then
					if row.Column == random_num4 then
						storePromotion(playerID, row.UnitPromotionType, row.PromotionClass);
						addLevelToPlayer(playerID, row.PromotionClass, row.Level);
						promotionChosen = true;
					end
				elseif level == 3 then
					if row.Column == random_num2 then
						storePromotion(playerID, row.UnitPromotionType, row.PromotionClass);
						addLevelToPlayer(playerID, row.PromotionClass, row.Level);
						promotionChosen = true;
					end
				else
					if row.Column == random_num7 then
						print("AI Earned Promotion: " .. row.UnitPromotionType);
						storePromotion(playerID, row.UnitPromotionType, row.PromotionClass);
						addLevelToPlayer(playerID, row.PromotionClass, row.Level);
						promotionChosen = true;
					end
				end
			
			end
		end
	end
end

function OnPlayerTurnActivated( playerID, bFirstTime )
		local pPlayer = Players[playerID];
		if (pPlayer:IsMajor() and pPlayer:IsHuman() == false) then
			local infantryDoctrineTotal = 0;
			local cavalryDoctrineTotal = 0;
			local reconDoctrineTotal = 0;
			local rangedDoctrineTotal = 0;
			local navalDoctrineTotal = 0;
			local airDoctrineTotal = 0;
			for i, xpItem in ipairs(ExposedMembers.doctrineXPTableEM) do
									rightGroup = true;
									for j, xpItem in ipairs(xpItem) do
											
											if j == 1 and (xpItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											if j == 2 and rightGroup then
												ExposedMembers.infantryXPTotalAI = xpItem;								
											end
											if j == 3 and rightGroup then
												ExposedMembers.cavalryXPTotalAI = xpItem;								
											end
											if j == 4 and rightGroup then
												ExposedMembers.reconXPTotalAI = xpItem;								
											end
											if j == 5 and rightGroup then
												ExposedMembers.rangedXPTotalAI = xpItem;								
											end
											if j == 6 and rightGroup then
												ExposedMembers.navalXPTotalAI = xpItem;								
											end
											if j == 7 and rightGroup then
												ExposedMembers.airXPTotalAI = xpItem;								
											end
									end								
				end
				for i, doctrineLevelItem in ipairs(ExposedMembers.doctrineLevelTableEM) do
									rightGroup = true;
									for j, doctrineLevelItem in ipairs(doctrineLevelItem) do
										
											if j == 1 and (doctrineLevelItem ~= playerID or playerID == nil) then
												--skip
												rightGroup = false;
											end
											-- Looping through all ArmyUnits
											if j == 2 and rightGroup then
												infantryDoctrineTotal = infantryDoctrineTotal + doctrineLevelItem;
												infantryLvl1Total = doctrineLevelItem;
											end
											if j == 3 and rightGroup then
												infantryDoctrineTotal = infantryDoctrineTotal + doctrineLevelItem;
												infantryLvl2Total = doctrineLevelItem;
											end
											if j == 4 and rightGroup then
												infantryDoctrineTotal = infantryDoctrineTotal + doctrineLevelItem;
												infantryLvl3Total = doctrineLevelItem;
											end
											
											if j == 4 and rightGroup then
												infantryDoctrineCost = (infantryDoctrineTotal + 1) * 30 + (infantryDoctrineTotal * 20);
												print("Doctrine Total: " .. infantryDoctrineTotal);
												print("Doctrine Cost: " .. infantryDoctrineCost);
												print("Doctrine Total lvl 1: " .. infantryLvl1Total);
												print("Doctrine Total lvl 2: " .. infantryLvl2Total);
												print("Doctrine Total lvl 3: " .. infantryLvl3Total);
												if infantryDoctrineCost <= ExposedMembers.infantryXPTotalAI then
													if  infantryLvl1Total >= 2 * (infantryLvl2Total + 1) then
														selectRandomDoctrine(playerID, 'I', 2);
													elseif infantryLvl2Total >= 2 * (infantryLvl3Total + 1) then
														selectRandomDoctrine(playerID, 'I', 3);
													else
														selectRandomDoctrine(playerID, 'I', 1);
													end
												end
											
											end
											if j == 5 and rightGroup then
												cavalryDoctrineTotal = cavalryDoctrineTotal + doctrineLevelItem;
												cavalryLvl1Total = doctrineLevelItem;
											end
											if j == 6 and rightGroup then
												cavalryDoctrineTotal = cavalryDoctrineTotal + doctrineLevelItem;
												cavalryLvl2Total = doctrineLevelItem;
											end
											if j == 7 and rightGroup then
												cavalryDoctrineTotal = cavalryDoctrineTotal + doctrineLevelItem;
												cavalryLvl3Total = doctrineLevelItem;
											end
											
											if j == 7 and rightGroup then
												cavalryDoctrineCost = (cavalryDoctrineTotal + 1) * 30 + (cavalryDoctrineTotal * 20);
												if cavalryDoctrineCost <= ExposedMembers.cavalryXPTotalAI then
													if  cavalryLvl1Total >= 2 * (cavalryLvl2Total + 1) and cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal then
														selectRandomDoctrine(playerID, 'C', 2);
													elseif cavalryLvl2Total >= 2 * (cavalryLvl3Total + 1) and cavalryDoctrineCost <= ExposedMembers.cavalryXPTotal then
														selectRandomDoctrine(playerID, 'C', 3);
													else
														selectRandomDoctrine(playerID, 'C', 1);
													end
												end
											
											end
											if j == 8 and rightGroup then
												reconDoctrineTotal = reconDoctrineTotal + doctrineLevelItem;
												reconLvl1Total = doctrineLevelItem;
											end
											if j == 9 and rightGroup then
												reconDoctrineTotal = reconDoctrineTotal + doctrineLevelItem;
												reconLvl2Total = doctrineLevelItem;
											end
											if j == 10 and rightGroup then
												reconDoctrineTotal = reconDoctrineTotal + doctrineLevelItem;
												reconLvl3Total = doctrineLevelItem;
											end
											
											if j == 10 and rightGroup then
												reconDoctrineCost = (reconDoctrineTotal + 1) * 30 + (reconDoctrineTotal * 20);
												if reconDoctrineCost <= ExposedMembers.reconXPTotalAI then
													if  reconLvl1Total >= 2 * (reconLvl2Total + 1) then
														selectRandomDoctrine(playerID, 'S', 2);
													elseif reconLvl2Total >= 2 * (reconLvl3Total + 1) then
														selectRandomDoctrine(playerID, 'S', 3);
													else
														selectRandomDoctrine(playerID, 'S', 1);
													end
												end
											
											end
											if j == 11 and rightGroup then
												rangedDoctrineTotal = rangedDoctrineTotal + doctrineLevelItem;
												rangedLvl1Total = doctrineLevelItem;
											end
											if j == 12 and rightGroup then
												rangedDoctrineTotal = rangedDoctrineTotal + doctrineLevelItem;
												rangedLvl2Total = doctrineLevelItem;
											end
											if j == 13 and rightGroup then
												rangedDoctrineTotal = rangedDoctrineTotal + doctrineLevelItem;
												rangedLvl3Total = doctrineLevelItem;
											end
											
											if j == 13 and rightGroup then
												rangedDoctrineCost = (rangedDoctrineTotal + 1) * 30 + (rangedDoctrineTotal * 20);
												if rangedDoctrineCost <= ExposedMembers.rangedXPTotalAI then
													if  rangedLvl1Total >= 2 * (rangedLvl2Total + 1) then
														selectRandomDoctrine(playerID, 'R', 2);
													elseif rangedLvl2Total >= 2 * (rangedLvl3Total + 1) then
														selectRandomDoctrine(playerID, 'R', 3);
													else
														selectRandomDoctrine(playerID, 'R', 1);
													end
												end
											
											end
											if j == 14 and rightGroup then
												navalDoctrineTotal = navalDoctrineTotal + doctrineLevelItem;
												navalLvl1Total = doctrineLevelItem;
											end
											if j == 15 and rightGroup then
												navalDoctrineTotal = navalDoctrineTotal + doctrineLevelItem;
												navalLvl2Total = doctrineLevelItem;
											end
											if j == 16 and rightGroup then
												navalDoctrineTotal = navalDoctrineTotal + doctrineLevelItem;
												navalLvl3Total = doctrineLevelItem;
											end
											
											if j == 16 and rightGroup then
												navalDoctrineCost = (navalDoctrineTotal + 1) * 30 + (navalDoctrineTotal * 20);
												if navalDoctrineCost <= ExposedMembers.navalXPTotalAI then
													if  navalLvl1Total >= 2 * (navalLvl2Total + 1) then
														selectRandomDoctrine(playerID, 'N', 2);
													elseif navalLvl2Total >= 2 * (navalLvl3Total + 1) then
														selectRandomDoctrine(playerID, 'N', 3);
													else
														selectRandomDoctrine(playerID, 'N', 1);
													end
												end
											
											end
											if j == 17 and rightGroup then
												airDoctrineTotal = airDoctrineTotal + doctrineLevelItem;
												airLvl1Total = doctrineLevelItem;
											end
											if j == 18 and rightGroup then
												airDoctrineTotal = airDoctrineTotal + doctrineLevelItem;
												airLvl2Total = doctrineLevelItem;
											end
											if j == 19 and rightGroup then
												airDoctrineTotal = airDoctrineTotal + doctrineLevelItem;
												airLvl3Total = doctrineLevelItem;
											end
											
											if j == 19 and rightGroup then
												airDoctrineCost = (airDoctrineTotal + 1) * 30 + (airDoctrineTotal * 20);
												if airDoctrineCost <= ExposedMembers.airXPTotalAI then
													if  airLvl1Total >= 2 * (airLvl2Total + 1) then
														selectRandomDoctrine(playerID, 'A', 2);
													elseif airLvl2Total >= 2 * (airLvl3Total + 1) then
														selectRandomDoctrine(playerID, 'A', 3);
													else
														selectRandomDoctrine(playerID, 'A', 1);
													end
												end
											
											end
									end								
				end
		end
end
Events.PlayerTurnActivated.Add( OnPlayerTurnActivated );



function Initialize()

	local iconName = "ICON_CIVIC_FUTURE_CIVIC";
	  local textureOffsetX, textureOffsetY, textureSheet = IconManager:FindIconAtlas(iconName,38);
	  if (textureOffsetX ~= nil) then
		Controls.ReportsImage:SetTexture( textureOffsetX, textureOffsetY, textureSheet );
	  end
	  
	ExposedMembers.DoctrinesTypeStatus = 'Infantry';
	ContextPtr:SetInputHandler( OnInputHandler, true );
	Controls.CultureButton:RegisterCallback(Mouse.eLClick, OnOpenCulture);
  Controls.CultureButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  Controls.CultureMeterButton:RegisterCallback(Mouse.eLClick, OnOpenCulture);
  Controls.GovernmentButton:RegisterCallback( Mouse.eLClick, OnGovernmentClick );
  Controls.GovernmentButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  Controls.GreatPeopleButton:RegisterCallback( Mouse.eLClick, OnOpenGreatPeople );
  Controls.GreatPeopleButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  Controls.GreatWorksButton:RegisterCallback( Mouse.eLClick, OnOpenGreatWorks );
  Controls.GreatWorksButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  Controls.ReligionButton:RegisterCallback( Mouse.eLClick, OnOpenReligion );
  Controls.ReligionButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  Controls.ScienceButton:RegisterCallback(Mouse.eLClick, OnOpenResearch);
  Controls.ScienceButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  Controls.ScienceMeterButton:RegisterCallback(Mouse.eLClick, OnOpenResearch);
	-- Doctrines
	Controls.DoctrinesButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
	Controls.DoctrinesButton:RegisterCallback(Mouse.eLClick, OnOpenDoctrines);
	Controls.CloseDoctrines:RegisterCallback(Mouse.eLClick, OnCloseDoctrines);
	Controls.ButtonInfantryDoctrine:RegisterCallback(Mouse.eLClick, OnInfantryDoctrine);
	Controls.ButtonCavalryDoctrine:RegisterCallback(Mouse.eLClick, OnCavalryDoctrine);
	Controls.ButtonReconDoctrine:RegisterCallback(Mouse.eLClick, OnReconDoctrine);
	Controls.ButtonNavalDoctrine:RegisterCallback(Mouse.eLClick, OnNavalDoctrine);
	Controls.ButtonAirDoctrine:RegisterCallback(Mouse.eLClick, OnAirDoctrine);
	Controls.ButtonRangedDoctrine:RegisterCallback(Mouse.eLClick, OnRangedDoctrine);
	
	 -- CQUI --
  Controls.ReportsButton:RegisterCallback(Mouse.eLClick, OnToggleReportsScreen);
  Controls.ReportsButton:RegisterCallback( Mouse.eMouseEnter, function() UI.PlaySound("Main_Menu_Mouse_Over"); end);
  -- CQUI --


	local MyGlobalsRef : table = {};
		LuaEvents.GetMyGlobals(MyGlobalsRef);
		MyGlobalsRef.MyGlobals.globalPromotionTable = ExposedMembers.promotionTableEM;
	

	Events.TurnBegin.Add( OnTurnBegin );
  Events.VisualStateRestored.Add( OnTurnBegin );
  Events.CivicCompleted.Add( OnCivicCompleted );				-- To capture when we complete Code of Laws
  Events.CivicChanged.Add(OnTurnBegin);
  Events.ResearchChanged.Add(OnTurnBegin);
  Events.TreasuryChanged.Add( RefreshGovernment );
  Events.GovernmentPolicyChanged.Add( RefreshGovernment );
  Events.GovernmentPolicyObsoleted.Add( RefreshGovernment );
  Events.GovernmentChanged.Add( RefreshGovernment );
  Events.AnarchyBegins.Add( RefreshGovernment );
  Events.AnarchyEnds.Add( RefreshGovernment );
  Events.InterfaceModeChanged.Add( OnInterfaceModeChanged );
  Events.GreatWorkCreated.Add( OnGreatWorkCreated );
  Events.FaithChanged.Add( OnFaithChanged );
  Events.LocalPlayerChanged.Add( OnLocalPlayerChanged );
  Events.DiplomacyDealEnacted.Add( OnDiplomacyDealEnacted );

  LuaEvents.CivicsTree_CloseCivicsTree.Add(SetCivicsTreeClosed);
  LuaEvents.CivicsTree_OpenCivicsTree.Add( SetCivicsTreeOpen );
  LuaEvents.Government_CloseGovernment.Add( SetGovernmentClosed );
  LuaEvents.Government_OpenGovernment.Add( SetGovernmentOpen );
  LuaEvents.GreatPeople_CloseGreatPeople.Add( SetGreatPeopleClosed );
  LuaEvents.GreatPeople_OpenGreatPeople.Add( SetGreatPeopleOpen );
  LuaEvents.GreatWorks_CloseGreatWorks.Add( SetGreatWorksClosed );
  LuaEvents.GreatWorks_OpenGreatWorks.Add( SetGreatWorksOpen );
  LuaEvents.Religion_CloseReligion.Add( SetReligionClosed );
  LuaEvents.Religion_OpenReligion.Add( SetReligionOpen );
  LuaEvents.TechTree_CloseTechTree.Add(SetTechTreeClosed);
  LuaEvents.TechTree_OpenTechTree.Add( SetTechTreeOpen );
  LuaEvents.Tutorial_CloseAllLaunchBarScreens.Add( OnTutorialCloseAll );

	if HasCapability("CAPABILITY_TECH_TREE") then
		LuaEvents.WorldTracker_ToggleResearchPanel.Add(OnToggleResearchPanel);
	end
	if HasCapability("CAPABILITY_CIVICS_TREE") then
		LuaEvents.WorldTracker_ToggleCivicPanel.Add(OnToggleCivicPanel);
	end

	OnTurnBegin();	
end
Initialize();
